@@ -1,16 +1,14 @@
 """Code common to Connection and Channel objects."""
 # Copyright (C) 2007-2008 Barry Pederson <bp@barryp.org>)
-from __future__ import absolute_import, unicode_literals
 
 import logging
 
 from vine import ensure_promise, promise
 
 from .exceptions import AMQPNotImplementedError, RecoverableConnectionError
-from .five import bytes_if_py2
 from .serialization import dumps, loads
 
-__all__ = ['AbstractChannel']
+__all__ = ('AbstractChannel',)
 
 AMQP_LOGGER = logging.getLogger('amqp')
 
@@ -19,7 +17,7 @@ Received method %s during closing channel %s. This method will be ignored\
 """
 
 
-class AbstractChannel(object):
+class AbstractChannel:
     """Superclass for Connection and Channel.
 
     The connection is treated as channel 0, then comes
@@ -54,7 +52,7 @@ class AbstractChannel(object):
         conn = self.connection
         if conn is None:
             raise RecoverableConnectionError('connection already closed')
-        args = dumps(format, args) if format else bytes_if_py2('')
+        args = dumps(format, args) if format else ''
         try:
             conn.frame_writer(1, self.channel_id, sig, args, content)
         except StopIteration:
@@ -122,7 +120,7 @@ class AbstractChannel(object):
             amqp_method = self._METHODS[method_sig]
         except KeyError:
             raise AMQPNotImplementedError(
-                'Unknown AMQP method {0!r}'.format(method_sig))
+                f'Unknown AMQP method {method_sig!r}')
 
         try:
             listeners = [self._callbacks[method_sig]]
