@@ -4,36 +4,17 @@
 
 """
 # Copyright (C) 2007 Barry Pederson <bp@barryp.org>
-#
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Lesser General Public
-# License as published by the Free Software Foundation; either
-# version 2.1 of the License, or (at your option) any later version.
-#
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
-from __future__ import absolute_import, unicode_literals
 
 import calendar
-import sys
-
 from datetime import datetime
 from decimal import Decimal
 from io import BytesIO
+from struct import pack, unpack_from
 
-from .spec import Basic
 from .exceptions import FrameSyntaxError
-from .five import int_types, long_t, string, string_t, items
-from .platform import pack, unpack_from
-from .utils import bytes_to_str as pstr_t, str_to_bytes
-
-ftype_t = chr if sys.version_info[0] == 3 else None
+from .spec import Basic
+from .utils import bytes_to_str as pstr_t
+from .utils import str_to_bytes
 
 ILLEGAL_TABLE_TYPE = """\
     Table type {0!r} not handled by amqp.
@@ -48,15 +29,19 @@ ILLEGAL_TABLE_TYPE_WITH_VALUE = """\
 """
 
 
-def _read_item(buf, offset=0, unpack_from=unpack_from, ftype_t=ftype_t):
-    ftype = ftype_t(buf[offset]) if ftype_t else buf[offset]
+def _read_item(buf, offset):
+    ftype = chr(buf[offset])
     offset += 1
 
     # 'S': long string
     if ftype == 'S':
         slen, = unpack_from('>I', buf, offset)
         offset += 4
-        val = pstr_t(buf[offset:offset + slen])
+        try:
+            val = pstr_t(buf[offset:offset + slen])
+        except UnicodeDecodeError:
+            val = buf[offset:offset + slen]
+
         offset += slen
     # 's': short string
     elif ftype == 's':
@@ -64,6 +49,12 @@ def _read_item(buf, offset=0, unpack_from=unpack_from, ftype_t=ftype_t):
         offset += 1
         val = pstr_t(buf[offset:offset + slen])
         offset += slen
+    # 'x': Bytes Array
+    elif ftype == 'x':
+        blen, = unpack_from('>I', buf, offset)
+        offset += 4
+        val = buf[offset:offset + blen]
+        offset += blen
     # 'b': short-short int
     elif ftype == 'b':
         val, = unpack_from('>B', buf, offset)
@@ -147,14 +138,12 @@ def _read_item(buf, offset=0, unpack_from=unpack_from, ftype_t=ftype_t):
         val = None
     else:
         raise FrameSyntaxError(
-            'Unknown value in table: {0!r} ({1!r})'.format(
+            'Unknown value in table: {!r} ({!r})'.format(
                 ftype, type(ftype)))
     return val, offset
 
 
-def loads(format, buf, offset=0,
-          ord=ord, unpack_from=unpack_from,
-          _read_item=_read_item, pstr_t=pstr_t):
+def loads(format, buf, offset):
     """Deserialize amqp format.
 
     bit = b
@@ -179,11 +168,11 @@ def loads(format, buf, offset=0,
         if p == 'b':
             if not bitcount:
                 bits = ord(buf[offset:offset + 1])
-            bitcount = 8
+                offset += 1
+                bitcount = 8
             val = (bits & 1) == 1
             bits >>= 1
             bitcount -= 1
-            offset += 1
         elif p == 'o':
             bitcount = bits = 0
             val, = unpack_from('>B', buf, offset)
@@ -208,14 +197,19 @@ def loads(format, buf, offset=0,
             bitcount = bits = 0
             slen, = unpack_from('B', buf, offset)
             offset += 1
-            val = buf[offset:offset + slen].decode('utf-8')
+            val = buf[offset:offset + slen].decode('utf-8', 'surrogatepass')
             offset += slen
         elif p == 'S':
             bitcount = bits = 0
             slen, = unpack_from('>I', buf, offset)
             offset += 4
-            val = buf[offset:offset + slen].decode('utf-8')
+            val = buf[offset:offset + slen].decode('utf-8', 'surrogatepass')
             offset += slen
+        elif p == 'x':
+            blen, = unpack_from('>I', buf, offset)
+            offset += 4
+            val = buf[offset:offset + blen]
+            offset += blen
         elif p == 'F':
             bitcount = bits = 0
             tlen, = unpack_from('>I', buf, offset)
@@ -248,7 +242,7 @@ def loads(format, buf, offset=0,
     return values, offset
 
 
-def _flushbits(bits, write, pack=pack):
+def _flushbits(bits, write):
     if bits:
         write(pack('B' * len(bits), *bits))
         bits[:] = []
@@ -256,7 +250,7 @@ def _flushbits(bits, write, pack=pack):
 
 
 def dumps(format, values):
-    """"Serialize AMQP arguments.
+    """Serialize AMQP arguments.
 
     Notes:
         bit = b
@@ -266,6 +260,7 @@ def dumps(format, values):
         long long = L
         shortstr = s
         longstr = S
+        byte array = x
         table = F
         array = A
     """
@@ -303,15 +298,15 @@ def dumps(format, values):
         elif p == 's':
             val = val or ''
             bitcount = _flushbits(bits, write)
-            if isinstance(val, string):
-                val = val.encode('utf-8')
+            if isinstance(val, str):
+                val = val.encode('utf-8', 'surrogatepass')
             write(pack('B', len(val)))
             write(val)
-        elif p == 'S':
+        elif p == 'S' or p == 'x':
             val = val or ''
             bitcount = _flushbits(bits, write)
-            if isinstance(val, string):
-                val = val.encode('utf-8')
+            if isinstance(val, str):
+                val = val.encode('utf-8', 'surrogatepass')
             write(pack('>I', len(val)))
             write(val)
         elif p == 'F':
@@ -321,18 +316,18 @@ def dumps(format, values):
             bitcount = _flushbits(bits, write)
             _write_array(val or [], write, bits)
         elif p == 'T':
-            write(pack('>Q', long_t(calendar.timegm(val.utctimetuple()))))
+            write(pack('>Q', int(calendar.timegm(val.utctimetuple()))))
     _flushbits(bits, write)
 
     return out.getvalue()
 
 
-def _write_table(d, write, bits, pack=pack):
+def _write_table(d, write, bits):
     out = BytesIO()
     twrite = out.write
-    for k, v in items(d):
-        if isinstance(k, string):
-            k = k.encode('utf-8')
+    for k, v in d.items():
+        if isinstance(k, str):
+            k = k.encode('utf-8', 'surrogatepass')
         twrite(pack('B', len(k)))
         twrite(k)
         try:
@@ -345,10 +340,10 @@ def _write_table(d, write, bits, pack=pack):
     write(table_data)
 
 
-def _write_array(l, write, bits, pack=pack):
+def _write_array(list_, write, bits):
     out = BytesIO()
     awrite = out.write
-    for v in l:
+    for v in list_:
         try:
             _write_item(v, awrite, bits)
         except ValueError:
@@ -359,21 +354,17 @@ def _write_array(l, write, bits, pack=pack):
     write(array_data)
 
 
-def _write_item(v, write, bits, pack=pack,
-                string_t=string_t, bytes=bytes, string=string, bool=bool,
-                float=float, int_types=int_types, Decimal=Decimal,
-                datetime=datetime, dict=dict, list=list, tuple=tuple,
-                None_t=None):
-    if isinstance(v, (string_t, bytes)):
-        if isinstance(v, string):
-            v = v.encode('utf-8')
+def _write_item(v, write, bits):
+    if isinstance(v, (str, bytes)):
+        if isinstance(v, str):
+            v = v.encode('utf-8', 'surrogatepass')
         write(pack('>cI', b'S', len(v)))
         write(v)
     elif isinstance(v, bool):
         write(pack('>cB', b't', int(v)))
     elif isinstance(v, float):
         write(pack('>cd', b'd', v))
-    elif isinstance(v, int_types):
+    elif isinstance(v, int):
         if v > 2147483647 or v < -2147483647:
             write(pack('>cq', b'L', v))
         else:
@@ -388,21 +379,20 @@ def _write_item(v, write, bits, pack=pack,
         write(pack('>cBi', b'D', -exponent, v))
     elif isinstance(v, datetime):
         write(
-            pack('>cQ', b'T', long_t(calendar.timegm(v.utctimetuple()))))
+            pack('>cQ', b'T', int(calendar.timegm(v.utctimetuple()))))
     elif isinstance(v, dict):
         write(b'F')
         _write_table(v, write, bits)
     elif isinstance(v, (list, tuple)):
         write(b'A')
         _write_array(v, write, bits)
-    elif v is None_t:
+    elif v is None:
         write(b'V')
     else:
         raise ValueError()
 
 
-def decode_properties_basic(buf, offset=0,
-                            unpack_from=unpack_from, pstr_t=pstr_t):
+def decode_properties_basic(buf, offset):
     """Decode basic properties."""
     properties = {}
 
@@ -479,7 +469,7 @@ PROPERTY_CLASSES = {
 }
 
 
-class GenericContent(object):
+class GenericContent:
     """Abstract base class for AMQP content.
 
     Subclasses should override the PROPERTIES attribute.
@@ -509,8 +499,7 @@ class GenericContent(object):
             return self.properties[name]
         raise AttributeError(name)
 
-    def _load_properties(self, class_id, buf, offset=0,
-                         classes=PROPERTY_CLASSES, unpack_from=unpack_from):
+    def _load_properties(self, class_id, buf, offset):
         """Load AMQP properties.
 
         Given the raw bytes containing the property-flags and property-list
@@ -518,7 +507,7 @@ class GenericContent(object):
         stored in this object as an attribute named 'properties'.
         """
         # Read 16-bit shorts until we get one with a low bit set to zero
-        props, offset = classes[class_id](buf, offset)
+        props, offset = PROPERTY_CLASSES[class_id](buf, offset)
         self.properties = props
         return offset
 
@@ -534,7 +523,6 @@ class GenericContent(object):
         flags = []
         sformat, svalues = [], []
         props = self.properties
-        props.setdefault('content_encoding', 'utf-8')
         for key, proptype in self.PROPERTIES:
             val = props.get(key, None)
             if val is not None:
